@echo off &setlocal
color f0

rem - Manuelle Vorbereitungen Anfang

rem - Lade Java https://java.com/de/download/windows_manual.jsp
rem - Installiere Java 

rem - Lade Splitter https://www.mkgmap.org.uk/download/splitter.html
rem - Lade Mkgmap https://www.mkgmap.org.uk/download/mkgmap.html
rem - Lade Bounds http://osm.thkukuk.de/data/bounds-latest.zip
rem - Lade Sea http://osm.thkukuk.de/data/sea-latest.zip
rem - Lade Style https://my.cloudme.com/#scotti/MotoRouteOpenStreetMap (mrosm-style-r024.zip)
rem - Alle Dateien bleiben gezipt und werden nicht entpackt!

rem - Lade gewuenschte Geodaten im Format osm.pbf https://download.geofabrik.de 

rem - Starte diese Batch-Datei
rem - Der Ordner dieser Batch-Datei wird als Projekt-Ordner verwendet!

rem - Waehle splitter-*.zip Pfad
rem - Waehle mkgmap-*.zip Pfad
rem - Waehle style-*.zip Pfad
rem - Waehle bounds-latest.zip Pfad
rem - Waehle sea-lastest.zip Pfad
rem - Waehle .obm.pbf Datei Pfad

rem - Benchmark:
rem - Hardware M1 MacBook Air (2020)
rem - Virtueller PC Windows 11 UTM/QEMU mit 3G Max Java Heap Space
rem - Germany auf interner SSD: Splitter ~20 Minuten, Mkgmap ~46 Minuten
rem - Germany auf USB3.0   HDD: Splitter ~60 Minuten, Mkgmap ~75 Minuten
rem ================================================================================

set varVersion=0.11
set varProductVersion=115
set varVersionDate=15.02.2023

:modTitle
title MotoRoute-OpenStreetMap (Backmischung light)

echo MotoRoute-OpenStreetMap (Backmischung light) 
echo Version %varVersion%.%varProductVersion% (fuer Windows, auch Windows 11 und ARM-CPU, auch UTM/QEMU virtualisiert)
echo (C) Gernot Skottke, %varVersionDate%
echo scotti@web.de
echo.
echo Folgende Eingaben werden verarbeitet:
echo.

:modJava
java -version 1> nul 2> javaversion.log

findstr /L /C:"version" javaversion.log > java.log

set /p varJava=<java.log
del java*.log

if not defined varJava (

	color cf
	
	echo MROSM: Java nicht gefunden
	echo.

	call :getBalloon "Java nicht gefunden" Hand MotoRoute-OpenStreetMap Error
	goto :Ende
) 

echo Java-Version   : %varJava%
echo Projekt-Ordner : "%~dp0"

:modSplitter
rem Preparation PowerShell command (.osm.pbf)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'Splitter (splitter-*.zip)| splitter-*.zip'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl splitter (zip-Datei)'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (.osm.pbf)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varSplitterPfad="%Filename%"

rem Abbruch geklickt
if [%varSplitterPfad%] == [" "] goto :Ende

rem Unzip
tar -xvzf %varSplitterPfad% -C "%~dp0\" 2> nul

cd splitter-* 2> nul

set varSplitterPfad=%CD%

if not exist "%varSplitterPfad%\splitter.jar" (

	color cf
	
	echo MROSM: Splitter nicht im erwarteten Ordner
	echo.

	call :getBalloon "Splitter nicht im erwarteten Ordner" Hand MotoRoute-OpenStreetMap Warning
	goto :Ende
)
cd %~dp0

echo Splitter-Ordner: "%varSplitterPfad%"

:modMkgmap
rem Preparation PowerShell command (.osm.pbf)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'Mkgmap (mkgmap-*.zip)| mkgmap-*.zip'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl mkgmap (zip-Datei)'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (.osm.pbf)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varMkgmapPfad="%Filename%"

rem Abbruch geklickt
if [%varMkgmapPfad%] == [" "] goto :Ende

rem Unzip
tar -xvzf %varMkgmapPfad% -C "%~dp0\" 2> nul

cd mkgmap-* 2> nul

set varMkgmapPfad=%CD%

if not exist "%varMkgmapPfad%\mkgmap.jar" (

	color cf

	echo MROSM: Mkgmap nicht im erwarteten Ordner
	echo.

	call :getBalloon "Mkgmap nicht im erwarteten Ordner" Hand MotoRoute-OpenStreetMap Warning
	goto :Ende
)
cd %~dp0

echo Mkgmap-Ordner  : "%varMkgmapPfad%"

:modStyle
rem Preparation PowerShell command (.osm.pbf)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'MROSM-Style (*style-*.zip)| *style-*.zip'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl MROSM-Style (zip-Datei)'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (.osm.pbf)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varStylePfad="%Filename%"

rem Abbruch geklickt
if [%varStylePfad%] == [" "] goto :Ende

rem Unzip
tar -xvzf %varStylePfad% -C "%~dp0\" 2> nul

cd style-* 2> nul

set varStylePfad=%CD%

if not exist "%varStylePfad%\mrosm.txt" (

	color cf
	
	echo MROSM: Style nicht im erwarteten Ordner
	echo.

	call :getBalloon "Style nicht im erwarteten Ordner" Hand MotoRoute-OpenStreetMap Warning
	goto :Ende
)
cd %~dp0

echo Style-Ordner   : "%varStylePfad%"

:modBounds
rem Preparation PowerShell command (bounds-latest.zip)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'Bounds (bounds*.zip)| bounds*.zip'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl Bounds'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (bounds-latest.zip)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varBounds="%Filename%"

rem Abbruch geklickt
if [%varBounds%] == [" "] goto :Ende

echo Bounds (Pfad)  : %varBounds%

:modSea
rem Preparation PowerShell command (sea-latest.zip)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'Sea (sea*.zip)| sea*.zip'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl Sea'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (bounds-latest.zip)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varSea="%Filename%"

rem Abbruch geklickt
if [%varSea%] == [" "] goto :Ende

echo Sea    (Pfad)  : %varSea%

:modOsmPbf
rem Preparation PowerShell command (.osm.pbf)
set pwshcmd=powershell -noprofile -command "&{[System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms') | Out-Null;$OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog; $OpenFileDialog.Multiselect = $false; $OpenFileDialog.Filter = 'OSM (*.osm.pbf)| *.osm.pbf'; $OpenFileDialog.InitialDirectory = [Environment]::GetFolderPath('Downloads'); $OpenFileDialog.Title = 'Auswahl .osm.pbf'; $OpenFileDialog.ShowDialog()|out-null; $OpenFileDialog.FileName}"

rem Exec commands powershell and get result in FileName variable (.osm.pbf)
for /f "delims=" %%I in ('%pwshcmd%') do set "Filename=%%I"

set varOsmPbf="%Filename%"

rem Abbruch geklickt
if [%varOsmPbf%] == [" "] goto :Ende

:modRegionName
rem %Filename1% mit 1 Endung .osm
for /f "delims=" %%i in ('echo %Filename%') do set "Filename1=%%~ni"
rem %Filename% ohne Endung
for /f "delims=" %%i in ('echo %Filename1%') do set "Filename=%%~ni"
rem Name (ohne "-latest") festlegen
setlocal enabledelayedexpansion
for %%a in ("a=A" "b=B" "c=C" "d=D" "e=E" "f=F" "g=G" "h=H" "i=I" "j=J" "k=K" "l=L"
 "m=M" "n=N" "o=O" "p=P" "q=Q" "r=R" "s=S" "t=T" "u=U" "v=V" "w=W" "x=X" "y=Y" "z=Z") do (set "Filename=!Filename:%%~a!") 
if [%Filename:~0,-7%] == [] (
	set varRegionName=%Filename%
) else if [%Filename:~-7%] == [-LATEST] (
	set varRegionName=%Filename:~0,-7%
) else (
	set varRegionName=%Filename%
)
rem mkgmap verarbeitet nur bis 50 Zeichen. Wir haben für die Region nur noch 27 frei.
set varRegionName=%varRegionName:~0,25%

:modRegionKurz
rem Region in Teil-Worte zerlegen (Rheinland-Pfalz in 1:Rheinland, 2:Pfalz)
for /f "tokens=1,2 delims=-" %%i in ("%varRegionName%") do set "varRegion1=%%i" &set "varRegion2=%%j"
if [%varRegion2%]==[] (
	rem Bayern = BAY
	set varRegion=%varRegion1:~0,3% 
) else (
	rem Rheinland-Pfalz = RP
	set varRegion=%varRegion1:~0,1%%varRegion2:~0,1% 
)

echo Region (Pfad)  : %varOsmPbf%
echo Region (Name)  : %varRegionName%
echo Region (kurz)  : %varRegion%

:modOsmPbfDate
if exist ./osmconvert64.exe (
	for /f "tokens=*" %%a in ('osmconvert64.exe %varOsmPbf% --out-timestamp') do set varOsmPbfDate=%%a
) 

if "%varOsmPbfDate%"=="" (
rem Datum = Heute
	set varD=%date:~-10,2%
	set varM=%date:~-7,2%
	set varY=%date:~-4%
) else (
rem Datum = Timestamp von osmconvert aus .osm.pbf
	set varD=%varOsmPbfDate:~8,2%
	set varM=%varOsmPbfDate:~5,2%
	set varY=%varOsmPbfDate:~0,4%
	
	echo osmconvert64   : "%~dp0osmconvert64.exe"
)

set varDate=%varY%-%varM%-%varD%

:modFamilyID
call :getDateToMJD %varY% %varM% %varD% MJD
if [%MJD:~2,1%]==[0] (
	set MJD3=5%MJD:~3,2%
) else (
	set MJD3=%MJD:~2,3%
)
set /a num=%random% %%9 +1
set varFid=%MJD3%%num%

echo Datum         : %varDate%
echo Family ID     : %varFid%

echo Productversion: %varProductVersion%

:modRam
rem System RAM ermitteln und in GB umrechnen
call :getRam
set /a varSystemRam=%varSystemRam: =%/1024/1024
set /a varSystemRam=%varSystemRam: =%-1

:modJavaHeapSpace
set varJavaMin=1G
set varJavaMax=%varSystemRam%G

echo Java Heap Max : %varJavaMax%

echo.

echo MROSM: Eine beliebige Taste druecken um fortzufahren. Schliessen um abzubrechen.

pause > nul

rem - Manuelle Vorbereitungen Ende
rem ================================================================================

rem Ordner fuer splitter 
if NOT exist "./tiles" (
	md "./tiles"
	echo MROSM: Ordner 'tiles' angelegt
) else (
	echo MROSM: Ordner 'tiles' vorhanden
)

rem Ordner fuer mkgmap 
if NOT exist "./tiles/gbasemap" (
	md "./tiles/gbasemap"
	echo MROSM: Ordner 'tiles/gbasemap' angelegt
) else (
	echo MROSM: Ordner 'tiles/gbasemap' vorhanden
) 

rem Ordner fuer maps 
if NOT exist "./maps" (
	md "./maps"
	echo MROSM: Ordner 'maps' angelegt
) else (
	echo MROSM: Ordner 'maps' vorhanden
) 

echo.

rem splitter 

color fc

echo MROSM: Start splitter
echo MROSM: %varOsmPbf%

rem splitter Befehl
echo.
echo MROSM: splitter.jar Befehl
echo ------
echo java -Xms%varJavaMin% -Xmx%varJavaMax% -ea -jar "%varSplitterPfad%\splitter.jar" ^
	--mapid=%varFid%0001 ^
	--output-dir=./tiles ^
	%varOsmPbf%
echo ------
echo.

java -Xms%varJavaMin% -Xmx%varJavaMax% -ea -jar "%varSplitterPfad%\splitter.jar" ^
	--mapid=%varFid%0001 ^
	--output-dir=./tiles ^
	%varOsmPbf%

echo.

echo MROSM: Ende splitter 

echo.

rem mkgmap 

color f9

echo MROSM: Start mkgmap

echo. 

rem Setze style-Pfad
cd style-*
set varStyle=%CD%
for %%i in ("%CD%") do set "varStyle=%%~nxi"
cd %~dp0

rem Setze mkgmap-Pfad
cd mkgmap-*
set varMkgmapPfad=%CD%
cd %~dp0

rem mkgmap Befehl
echo MROSM: mkgmap.jar Befehl
echo ------
echo java -Xms%varJavaMin% -Xmx%varJavaMax% -ea -jar "%varMkgmapPfad%\mkgmap.jar" ^
	--bounds=%varBounds% ^
	--precomp-sea=%varSea% ^
	--output-dir=./tiles/gbasemap ^
	--max-jobs ^
	--style-file=./%varStyle%/mrosm_style/ ^
	--description="MROSM %varRegionName% %varDate% %varFid%" ^
	--country-name=%varRegionName% ^
	--country-abbr=%varRegion% ^
	--family-id=%varFid% ^
	--product-id=1 ^
	--product-version=%varProductVersion% ^
	--mapname=%varFid%0001 ^
	--series-name="MROSM %varRegionName% %varDate% %varFid%" ^
	--family-name="MROSM-%varRegionName%-%varDate%-%varFid%" ^
	--area-name=%varRegion% ^
	--latin1 ^
	--draw-priority=10 ^
	--tdbfile ^
	--add-pois-to-areas ^
	--poi-address ^
	--link-pois-to-ways ^
	--add-pois-to-lines ^
	--housenumbers ^
	--levels=0:24,1:22,2:21,3:19,4:18,5:16 ^
	--location-autofill=is_in,nearest ^
	--merge-lines ^
	--index ^
	--x-split-name-index ^
	--net ^
	--route ^
	--remove-short-arcs ^
	--remove-ovm-work-files ^
	--gmapi ^
 	./tiles/*.osm.pbf ^
	--keep-going ^
	./%varStyle%/mrosm.txt
echo ------
rem Kann oben wieder eingefügt werden wenn .img File benötigt wird
rem --gmapsupp ^
echo.

java -Xms%varJavaMin% -Xmx%varJavaMax% -ea -jar "%varMkgmapPfad%\mkgmap.jar" ^
	--bounds=%varBounds% ^
	--precomp-sea=%varSea% ^
	--output-dir=./tiles/gbasemap ^
	--max-jobs ^
	--style-file=./%varStyle%/mrosm_style/ ^
	--description="MROSM %varRegionName% %varDate% %varFid%" ^
	--country-name=%varRegionName% ^
	--country-abbr=%varRegion% ^
	--family-id=%varFid% ^
	--product-id=1 ^
	--product-version=%varProductVersion% ^
	--mapname=%varFid%0001 ^
	--series-name="MROSM %varRegionName% %varDate% %varFid%" ^
	--family-name="MROSM-%varRegionName%-%varDate%-%varFid%" ^
	--area-name=%varRegion% ^
	--latin1 ^
	--draw-priority=10 ^
	--tdbfile ^
	--add-pois-to-areas ^
	--poi-address ^
	--link-pois-to-ways ^
	--add-pois-to-lines ^
	--housenumbers ^
	--levels=0:24,1:22,2:21,3:19,4:18,5:16 ^
	--location-autofill=is_in,nearest ^
	--merge-lines ^
	--index ^
	--x-split-name-index ^
	--net ^
	--route ^
	--remove-short-arcs ^
	--remove-ovm-work-files ^
	--gmapi ^
	--gmapsupp ^
 	./tiles/*.osm.pbf ^
	--keep-going ^
	./%varStyle%/mrosm.txt
	
echo. 

echo MROSM: Ende mkgmap

echo.

rem Cleanup

color f2

cd %~dp0

if exist ./tiles/gbasemap/gmapsupp.img (

	call :getBalloon "%varRegionName% frisch gebacken" Information MotoRoute-OpenStreetMap Info

	del .\tiles\gbasemap\%varFid%*.img
	del .\tiles\gbasemap\osm*.*
	del .\tiles\gbasemap\mrosm.typ
	
	call :getInst
	call :getUninst

	move .\tiles\gbasemap .\maps > nul	
	ren .\maps\gbasemap "%varRegionName%-%varDate%-%varFid%"
	
	rmdir .\tiles /s /q
	rmdir "%varSplitterPfad%" /s /q
	rmdir "%varMkgmapPfad%" /s /q
	rmdir "%varStylePfad%" /s /q
	
) else (

	call :getBalloon "Ups, hier ging was schief" Hand MotoRoute-OpenStreetMap Warning

)

echo MROSM: MotoRoute wuenscht gute Fahrt!

echo.

:Ende
echo MROSM: Zum Beenden bitte eine beliebige Taste druecken!

pause > nul

exit /b

:getInst
echo @echo off >> ./tiles/gbasemap/Install.bat
echo title Install MROSM-%varRegionName%-%varDate%-%varFid% >> ./tiles/gbasemap/Install.bat
echo echo. >> ./tiles/gbasemap/Install.bat
rem /d bedeutet nur neuere Dateien
rem /e Unterverzeichnisse mitnehmen
rem /y Dateien ohne Nachfragen überschreiben
rem /i neuen Ordner anlegen, falls nicht vorhanden
echo xcopy /eyi MROSM-%varRegionName%-%varDate%-%varFid%.gmap "%appdata%\Garmin\Maps\MROSM-%varRegionName%-%varDate%-%varFid%.gmap" >> ./tiles/gbasemap/Install.bat
echo echo. >> ./tiles/gbasemap/Install.bat
echo echo '%varRegionName%-%varDate%-%varFid%' kopiert nach '%appdata%\Garmin\Maps' >> ./tiles/gbasemap/Install.bat
echo echo. >> ./tiles/gbasemap/Install.bat
echo echo Gute Fahrt mit MotoRoute >> ./tiles/gbasemap/Install.bat
echo echo. >> ./tiles/gbasemap/Install.bat
echo color 2a >> ./tiles/gbasemap/Install.bat
echo pause >> ./tiles/gbasemap/Install.bat
exit /b

:getUninst
echo @echo off >> ./tiles/gbasemap/Uninstall.bat
echo title Uninstall MROSM-%varRegionName%-%varDate%-%varFid% >> ./tiles/gbasemap/Uninstall.bat
echo echo. >> ./tiles/gbasemap/Uninstall.bat
echo if exist "%appdata%\Garmin\Maps\MROSM-%varRegionName%-%varDate%-%varFid%.gmap" ( >> ./tiles/gbasemap/Uninstall.bat
echo     rmdir /s /q "%appdata%\Garmin\Maps\MROSM-%varRegionName%-%varDate%-%varFid%.gmap" >> ./tiles/gbasemap/Uninstall.bat
echo     echo '%varRegionName%-%varDate%-%varFid%' entfernt aus '%appdata%\Garmin\Maps' >> ./tiles/gbasemap/Uninstall.bat
echo     echo. >> ./tiles/gbasemap/Uninstall.bat
echo     echo Auf bald mit MotoRoute >> ./tiles/gbasemap/Uninstall.bat
echo     color 2e >> ./tiles/gbasemap/Uninstall.bat
echo ) else ( >> ./tiles/gbasemap/Uninstall.bat
echo     color 4e >> ./tiles/gbasemap/Uninstall.bat
echo     echo Diese Karte ist nicht mehr installiert. >> ./tiles/gbasemap/Uninstall.bat
echo ) >> ./tiles/gbasemap/Uninstall.bat
echo echo. >> ./tiles/gbasemap/Uninstall.bat
echo pause >> ./tiles/gbasemap/Uninstall.bat
exit /b

:getDateToMJD %yy% %mm% %dd% MJD
rem By Ritchie Lawrence, 2002-06-15. Version 1.0
rem Func: Returns a Modified Julian Day (MJD) from a UTC date. Reference date
rem (day 0) is Wednesday 17th November 1858. For NT4/2K/XP
rem Args: %1 year component used to create MJD, 2 or 4 digits (by val)
rem %2 month component used to create MJD, leading zero ok (by val)
rem %3 day of month used to create MJD, leading zero ok (by val)
rem %4 var to receive MJD (by ref)
setlocal ENABLEEXTENSIONS
set yy=%1&set mm=%2&set dd=%3
if 1%yy% LSS 200 if 1%yy% LSS 170 (set yy=20%yy%) else (set yy=19%yy%)
set /a dd=100%dd%%%100,mm=100%mm%%%100
set /a z=14-mm,z/=12,y=yy+4800-z,m=mm+12*z-3,j=153*m+2
set /a j=j/5+dd+y*365+y/4-y/100+y/400-2432046
endlocal & set %4=%j% & goto :eof

:getRam
for /f "skip=1" %%p in ('wmic os get TotalVisibleMemorySize') do ( 
   set varSystemRam=%%p
   goto :getRamEnd
)
:getRamEnd
exit /b

:getBalloon %Text% %Icon% %Title% %IconBalloon%
setlocal

if "%~2"=="" (set Icon=Information) else (set Icon=%2)
powershell -Command "[void] [System.Reflection.Assembly]::LoadWithPartialName('System.Windows.Forms'); $objNotifyIcon=New-Object System.Windows.Forms.NotifyIcon; $objNotifyIcon.BalloonTipText='%~1'; $objNotifyIcon.Icon=[system.drawing.systemicons]::%Icon%; $objNotifyIcon.BalloonTipTitle='%~3'; $objNotifyIcon.BalloonTipIcon='%~4'; $objNotifyIcon.Visible=$True; $objNotifyIcon.ShowBalloonTip(5000);"

endlocal

goto :eof